#include "system.h"			// system configuration file
#include <filter.h>


extern volatile short* rin_current_half[];
extern volatile short* sin_current_half[];
extern volatile short* rout_current_half[];
extern volatile short* sout_current_half[];

void Process_Data(void)	{
	int i,j;
	
	short receive_half;
	short transmit_half;
				
	volatile short*	copy_pointer_receive;
	volatile short*	copy_pointer_transmit;

	volatile short*	current_channel_rin;
	volatile short*	current_channel_sin;
	volatile short*	current_channel_sout;

				// do processing	
				
#ifdef	DEBUG_ALGORITHM
				*pFIO_FLAG_S = (1<<EXECUTION_TIME_ALL_CHANNELS);		// set flag
#endif

				// calcuate the half of the input and output buffers that can be processed and updated in this cycle
				receive_half= (Number_of_received_blocks == (Number_of_received_blocks>>1)<<1) ? 1 : 0;			// just received even or odd? 
				transmit_half= (Number_of_transmitted_blocks == (Number_of_transmitted_blocks>>1)<<1) ? 1 : 0;	// even or odd NOT being transmitted? 

				// copy RIN to ROUT (this is always done)
				copy_pointer_receive = rin_current_half[receive_half];
				copy_pointer_transmit = rout_current_half[transmit_half];
				for (i=0; i< Number_of_Channels * Number_of_Samples; i++)
					*copy_pointer_transmit++ = *copy_pointer_receive++;
				
				switch (Global_Status) {
				case PASS_THROUGH:
				// copy SIN to SOUT as well
					copy_pointer_receive = sin_current_half[receive_half];
					copy_pointer_transmit = sout_current_half[transmit_half];
					for (i=0; i< Number_of_Channels * Number_of_Samples; i++)
						*copy_pointer_transmit++ = *copy_pointer_receive++;
					break;
					
				case ENABLED:
#ifdef SOFTWARE_COMPANDING
					// expand input TDM samples according to the companding law	(RIN and SIN)
					(Software_Companding == SOFTWARE_MU_LAW) ? 	mu_expand((const short *)rin_current_half[receive_half], (short *)rin_current_half[receive_half], Number_of_Channels * Number_of_Samples *2) : \
																a_expand((const short *)rin_current_half[receive_half], (short *)rin_current_half[receive_half], Number_of_Channels * Number_of_Samples *2);
#endif	
					// process each channel according to its setting
					for (i=0; i< Number_of_Channels; i++)
						{	if (Per_Channel_Status[i] == OFF)
								{	// do a copy of this channel from SIN to SOUT
									copy_pointer_receive = sin_current_half[receive_half] + i;
									copy_pointer_transmit = sout_current_half[transmit_half] + i;
									for (j=0; j< Number_of_Samples; j++)
										*(copy_pointer_transmit + j * Number_of_Channels) = *(copy_pointer_receive + j * Number_of_Channels);
								}	// if (Per_Channel_Status[i] == OFF)
							else
								{	// apply the algorithm to this channel
									current_channel_rin = rin_current_half[receive_half] + i;
									current_channel_sin = sin_current_half[receive_half] + i;
									current_channel_sout = sout_current_half[receive_half] + i;
									// do the input scaling
									for (j=0; j< Number_of_Samples; j++) {
										*(current_channel_rin + j * Number_of_Channels) <<= RIN_scaling_factor[i];
										*(current_channel_sin + j * Number_of_Channels) <<= SIN_scaling_factor[i];
									}

									//*******************************************************************
									// CALL your algorithm here
									// The template presented here assumes that it
									// processes the RIN and SIN buffers and outputs the result 
									// to the SOUT buffers
									//*******************************************************************
									
									// do the output scaling (SOUT only, in the opposite direction)
									for (j=0; j< Number_of_Samples; j++) {
										*(current_channel_sout + j * Number_of_Channels) >>= SIN_scaling_factor[i];
									}

								}	// else
					}	// for (i=0; i< Number_of_Channels; i++)

#ifdef SOFTWARE_COMPANDING
					// at last compress output TDM samples according to the companding law	(SOUT only, since ROUT is already copied)
					(Software_Companding == SOFTWARE_MU_LAW) ? 	mu_compress((const short *)sout_current_half[transmit_half], (short *)sout_current_half[transmit_half], Number_of_Channels * Number_of_Samples) : \
																a_compress((const short *)sout_current_half[transmit_half], (short *)sout_current_half[transmit_half], Number_of_Channels * Number_of_Samples);
#endif	
					break;
					
				default: exit(1);
				}	// switch (Global_Status)
					
					
#ifdef	DEBUG_ALGORITHM
				*pFIO_FLAG_C = (1<<EXECUTION_TIME_ALL_CHANNELS);		// clear flag
#endif


}
